/**
 * @file
 * test.adjustColorBrightness node implementation.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

#include "VuoImageRenderer.h"

VuoModuleMetadata({
                     "title" : "Pulse Filter",
                     "description" : "Port of 'Pulse' by Danguafer/Silexars(2010), with adjustable strength and brightness, filter design. 2010-2014, George Toledo.",
					 "keywords" : [ ],
					 "version" : "1.0.0",
					 "dependencies" : [
						 "VuoImageRenderer"
					 ],
					 "node": {
						 "isInterface" : false
					 }
				 });

#include "node.h"

static const char * fragmentShaderSource = VUOSHADER_GLSL_SOURCE(120,
	// Inputs
	uniform sampler2D texture;
    uniform float time;
    uniform float ResX;
    uniform float ResY;
    uniform float X;
    uniform float Y;
    uniform float brightness;
    uniform float strength;

   // uniform vec4 mouse;
   // uniform sampler2D tex0;
    varying vec4 fragmentTextureCoordinate;

    void main(void)
    {
        vec2 resolution=vec2(ResX,ResY);
        vec2 halfres = resolution.xy/2.0;
        vec2 cPos = fragmentTextureCoordinate.xy*resolution;

        cPos.x -= X*resolution.x;//0.5*halfres.x*sin(time/2.0)+0.3*halfres.x*cos(time)+halfres.x;
        cPos.y -= Y*resolution.y;//0.4*halfres.y*sin(time/5.0)+0.3*halfres.y*cos(time)+halfres.y;
        float cLength = length(cPos);

        vec2 uv = (fragmentTextureCoordinate.xy+(cPos/cLength)*sin(cLength/30.0-time*10.0)/(1./(strength*.5)));
        vec3 col = texture2D(texture,uv).xyz*(brightness*50.)/cLength;

        gl_FragColor = vec4(col,1.0);
    }
);
struct nodeInstanceData
{
	VuoShader shader;
	VuoGlContext glContext;
	VuoImageRenderer imageRenderer;
};

struct nodeInstanceData * nodeInstanceInit(void)
{
	struct nodeInstanceData * instance = (struct nodeInstanceData *)malloc(sizeof(struct nodeInstanceData));
	VuoRegister(instance, free);

	instance->glContext = VuoGlContext_use();

    instance->shader = VuoShader_make("Pulse Deform Shader");
	VuoShader_addSource(instance->shader, VuoMesh_IndividualTriangles, NULL, NULL, fragmentShaderSource);
	VuoRetain(instance->shader);

	instance->imageRenderer = VuoImageRenderer_make(instance->glContext);
	VuoRetain(instance->imageRenderer);

	return instance;
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) instance,
		VuoInputData(VuoImage) image,
        VuoInputData(VuoReal, {"default":0,"suggestedStep":0.01}) time,
        VuoInputData(VuoReal, {"default":0.5,"suggestedStep":0.01}) X,
        VuoInputData(VuoReal, {"default":0.5,"suggestedStep":0.01}) Y,
        VuoInputData(VuoReal, {"default":1.0,"suggestedStep":0.1}) brightness,
        VuoInputData(VuoReal, {"default":0.1,"suggestedStep":0.01}) strength,

		VuoOutputData(VuoImage) adjustedImage
)
{
	if (! image)
		return;

	// Feed parameters to the shader.
	VuoShader_setUniform_VuoImage((*instance)->shader, "texture", image);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "time",     time);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "ResX",   image->pixelsWide);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "ResY",   image->pixelsHigh);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "X",   X);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "Y",   Y);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "brightness",  brightness);
    VuoShader_setUniform_VuoReal ((*instance)->shader, "strength", strength);

	// Render.
	*adjustedImage = VuoImageRenderer_draw((*instance)->imageRenderer, (*instance)->shader, image->pixelsWide, image->pixelsHigh, VuoImage_getColorDepth(image));
}

void nodeInstanceFini(VuoInstanceData(struct nodeInstanceData *) instance)
{
	VuoRelease((*instance)->shader);
	VuoRelease((*instance)->imageRenderer);
	VuoGlContext_disuse((*instance)->glContext);
}
